import React, { useCallback } from 'react';
import { useDispatch } from 'react-redux';
import { Maximize2 as Maximise } from 'styled-icons/feather/Maximize2';
import { Minimize2 as Minimise } from 'styled-icons/feather/Minimize2';
import { Box, Segment, Text, Icon } from '../../elements';
import { Loader } from '../../modules';
import { useAsyncComponent, useDashboardData } from '../../../hooks';
import { Menu, MenuItem } from '../Menu';
import { useConfig, useDashboardContext } from './context';
const buildButtonProps = (config, dispatch) => {
    const props = {};
    if (config.action) {
        props.onClick = () => {
            if (config.action) {
                dispatch(config.action);
            }
        };
    }
    return props;
};
const buildProps = (config, dispatch) => {
    switch (config.as) {
        case 'Button':
            return buildButtonProps(config, dispatch);
        default:
            return {};
    }
};
const StoredData = ({ config }) => {
    const { store } = useConfig();
    const data = useDashboardData(config && config.path, store);
    if (!data) {
        return React.createElement("span", null, config.fallback || 'Not found');
    }
    switch (config.type) {
        case 'string':
            return React.createElement("span", null, data);
    }
};
const WidgetContent = ({ config }) => {
    const dispatch = useDispatch();
    const Component = useAsyncComponent(config.as);
    const props = buildProps(config, dispatch);
    return (React.createElement(Component, Object.assign({}, props), config.data ? React.createElement(StoredData, { config: config.data }) : config.content));
};
export const WidgetMenu = ({ config, color, onMaximise, isMaximised }) => {
    // Don't render if no content
    if (config.title && !onMaximise) {
        return null;
    }
    return (React.createElement(Menu, null,
        config.title && (React.createElement(MenuItem, null,
            React.createElement(Text, { as: "span", color: color }, config.title))),
        onMaximise && (React.createElement(MenuItem, { position: "right" },
            React.createElement(Icon, { height: 12, width: 12, onClick: onMaximise }, isMaximised ? React.createElement(Minimise, null) : React.createElement(Maximise, null))))));
};
export const Widget = ({ config, id, }) => {
    const { color } = useConfig();
    const { maximised, setMaximised } = useDashboardContext();
    const size = maximised === id ? 1 : config.size;
    const toggleMaximise = useCallback(() => {
        if (maximised === id) {
            setMaximised('');
        }
        else {
            setMaximised(id);
        }
    }, [maximised, setMaximised, id]);
    return (React.createElement(Box, { size: size, padded: true, stretched: true },
        React.createElement(Segment, { raised: true },
            React.createElement(WidgetMenu, { config: config, color: color, onMaximise: toggleMaximise, isMaximised: maximised === id }),
            React.createElement(React.Suspense, { fallback: React.createElement(Loader, { active: true }) },
                React.createElement(WidgetContent, { config: config })))));
};
